package pro.litvinovg.w2phtml.gui;

import java.awt.Component;
import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.sun.star.uno.XComponentContext;

import w2phtml.Application;
import w2phtml.rdf.Metadata;

public class ConversionExecutor {
	
	HashMap<String,String> options = new HashMap<String,String>();
	private static final Logger logger = LoggerFactory.getLogger(ConversionExecutor.class);
	private String format = null;
	private String inputFilePath = null;
	private String outputFilePath = null;
	private String metadataFilePath = null;
	private JFrame frame;

	public ConversionExecutor(HashMap<String, Component> configuration, JFrame frame) {
		this.frame = frame;
		readOptions(configuration);
	}

	private void readOptions(HashMap<String, Component> configuration) {
		Set<String> keys = configuration.keySet();
		for (String optionName : keys) {
			String optionValue = null;
			Component component = configuration.get(optionName);
			if (component.getClass().equals(JTextField.class)) {
				optionValue = ((JTextField) component).getText();
			} else if (component.getClass().equals(JCheckBox.class)) {
				optionValue = Boolean.toString(((JCheckBox) component).isSelected());
			} else if (component.getClass().equals(JLabel.class)) {
				optionValue = ((JLabel) component).getText();
			} else if (component.getClass().equals(JComboBox.class)) {
				optionValue = ((JComboBox) component).getSelectedItem().toString();
			}
			options.put(optionName, optionValue);

		}
		format = options.get("targetFormat");
		options.remove("targetFormat");
		inputFilePath = options.get("inputFile");
		options.remove("inputFile");
		outputFilePath = options.get("outputFile");
		options.remove("outputFile");
		metadataFilePath = options.get("csv_metadata");
		options.remove("csv_metadata");

	}

	public String[] prepareArgs() {
		List<String> args = new ArrayList<String>();
		args.add("-"+format);
		for (String key: options.keySet()) {
			String value = options.get(key);
			if( value != null && !value.isEmpty()) {
				args.add("-"+key);
				args.add(value);
			}
		}
		if (metadataFilePath != null) {
			args.add("-csv_metadata");
			args.add(metadataFilePath);
		}
		args.add(inputFilePath);
		if (outputFilePath != null) {
			args.add(outputFilePath);
		}
		String[] array = args.toArray(new String[0]);
		return array; 
	}

	public boolean isConfigValid() {
		if (format == null) {
			return false;
		}
		File inputFile = new File(inputFilePath);
		File outputFile = null;
		
		if (!inputFile.exists()) {
			JOptionPane.showMessageDialog(frame, "Input file not exists");
			return false;
		}
		
		if (!inputFile.canRead()){
			JOptionPane.showMessageDialog(frame, "Input file can not be read");
			return false;
		}
		
		File metadataFile = null;
		if (metadataFilePath != null && !metadataFilePath.trim().isEmpty()) {
			metadataFile = new File(metadataFilePath);
			if (!metadataFile.exists()) {
				JOptionPane.showMessageDialog(frame, "Metadata file not exists");
				return false;
			}
			if (!metadataFile.canRead()) {
				JOptionPane.showMessageDialog(frame, "Metadata file can not be read");
				return false;
			}
		}
		if (outputFilePath == null || outputFilePath.trim().isEmpty()) {
			File inputDir = inputFile.getParentFile();
			if (!inputDir.canWrite()) {
				JOptionPane.showMessageDialog(frame, "Output directory is not writable");
				return false;
			}
		} else {
			outputFile = new File(outputFilePath);
			if (outputFile.exists()) {
				if (!outputFile.canWrite()) {
					if (outputFile.isDirectory()) {
						JOptionPane.showMessageDialog(frame, "Output directory is not writable");	
					} else {
						JOptionPane.showMessageDialog(frame, outputFile.exists() + "Output file is not writable");
					}
					return false;	
				}
			} else {
				try {
					outputFile.createNewFile();
					outputFile.delete();
				} catch (IOException e) {
					JOptionPane.showMessageDialog(frame, "Output file can not be created");	
					return false;
				}
			}
		}
		
		if (inputFile.isDirectory()) {
			if (isAFile(outputFile)){
				JOptionPane.showMessageDialog(frame, "Can't convert multiple inputs in one output");	
				return false;
			}
			if (isAFile(metadataFile)){
				JOptionPane.showMessageDialog(frame, "Can't convert multiple inputs with one metadata");	
				return false;
			}
		} else {
			if (metadataFile != null && metadataFile.isDirectory()){
				JOptionPane.showMessageDialog(frame, "Can't convert one input file with multiple metadata files");	
				return false;
			}
		}
		
		return true;
	}

	private boolean isAFile(File file) {
		return file != null && (!file.exists() || !file.isDirectory());
	}


	public void convert() {
		if (!isConfigValid()) {
			return;
		}
		File inputFile = new File(inputFilePath);
		if (inputFile.isDirectory()) {
			convertAllInDir();
		} else {
			tryConversion();	
			showResultsDialog();
			//JOptionPane.showMessageDialog(frame, "Conversion completed.");	
		}
		
	}

	private void showResultsDialog() {
		ResultsDialog results = new ResultsDialog();
		results.setModal(true);
		results.setVisible(true);
	}

	private void tryConversion() {
		try {
			Application.main(prepareArgs());	
		} catch (Exception e ) {
			logger.error("Conversion failed.");
			logger.error("Params: " + Arrays.toString(prepareArgs()));
		}
		
	}

	private void convertAllInDir() {
		File inputDir = new File(inputFilePath);
		if (!inputDir.isDirectory()) {
			return;
		}
		File[] inputFiles = inputDir.listFiles(new FilenameFilter() {
			@Override
			public boolean accept(File dir, String name) {
				return name.toLowerCase().endsWith("odt");
			}
		});
		if (inputFiles.length == 0) {
			JOptionPane.showMessageDialog(frame, "No ODT files in directory found.");
			return;
		}
		String outputPath = outputFilePath;
		String metadataDir = metadataFilePath;
		for (File inputFile : inputFiles) {
			convertFileInDir(inputFile, outputPath, metadataDir);
		}
		showResultsDialog();
	}

	private void convertFileInDir(File inputFile, String outputPath, String metadataDir) {
		String inputFileName = inputFile.getName();
		//remove extension
		inputFileName = inputFileName.substring(0, inputFileName.length() - 3);
		if ("rdf".equals(format)){
			String metadataInputFilePath = null;
			if (metadataDir != null && !metadataDir.trim().isEmpty()) {
				metadataInputFilePath = metadataDir + File.separator + inputFileName + "csv";
			} else {
				metadataInputFilePath = inputFile.getParent() + File.separator + inputFileName + "csv";				
			}
			File metadataFile = new File(metadataInputFilePath);
			if (metadataFile.exists() && metadataFile.canRead()) {
				metadataFilePath = metadataFile.getAbsolutePath();
			} else {
				metadataFilePath = null;
			}
		}
		if (outputPath != null && !outputPath.isEmpty()) {
			outputFilePath = outputPath + File.separator + inputFileName + getOutputExtension();
		}
		
		inputFilePath = inputFile.getAbsolutePath();
		tryConversion();		
		
	}

	private String getOutputExtension() {
		if (format.equals("rdf")) {
			return "rdf";
		} else if (format.equals("epub3")){
			return "epub";
		} else {
			return "html";
		}
	}
}
